# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..gtract import compareTractInclusion


def test_compareTractInclusion_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        closeness=dict(
            argstr="--closeness %f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        numberOfPoints=dict(
            argstr="--numberOfPoints %d",
        ),
        numberOfThreads=dict(
            argstr="--numberOfThreads %d",
        ),
        standardFiber=dict(
            argstr="--standardFiber %s",
            extensions=None,
        ),
        testFiber=dict(
            argstr="--testFiber %s",
            extensions=None,
        ),
        testForBijection=dict(
            argstr="--testForBijection ",
        ),
        testForFiberCardinality=dict(
            argstr="--testForFiberCardinality ",
        ),
        writeXMLPolyDataFile=dict(
            argstr="--writeXMLPolyDataFile ",
        ),
    )
    inputs = compareTractInclusion.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_compareTractInclusion_outputs():
    output_map = dict()
    outputs = compareTractInclusion.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Attempt to check each interface in nipype
"""
# Stdlib imports
import os
import re
import sys
import warnings

import black


# Functions and classes
class InterfaceChecker(object):
    """Class for checking all interface specifications
    """

    def __init__(
        self,
        package_name,
        package_skip_patterns=None,
        module_skip_patterns=None,
        class_skip_patterns=None,
    ):
        r""" Initialize package for parsing

        Parameters
        ----------
        package_name : string
            Name of the top-level package.  *package_name* must be the
            name of an importable package
        package_skip_patterns : None or sequence of {strings, regexps}
            Sequence of strings giving URIs of packages to be excluded
            Operates on the package path, starting at (including) the
            first dot in the package path, after *package_name* - so,
            if *package_name* is ``sphinx``, then ``sphinx.util`` will
            result in ``.util`` being passed for earching by these
            regexps.  If is None, gives default. Default is:
            ['\.tests$']
        module_skip_patterns : None or sequence
            Sequence of strings giving URIs of modules to be excluded
            Operates on the module name including preceding URI path,
            back to the first dot after *package_name*.  For example
            ``sphinx.util.console`` results in the string to search of
            ``.util.console``
            If is None, gives default. Default is:
            ['\.setup$', '\._']
        class_skip_patterns : None or sequence
            Sequence of strings giving classes to be excluded
            Default is: None

        """
        if package_skip_patterns is None:
            package_skip_patterns = ["\\.tests$"]
        if module_skip_patterns is None:
            module_skip_patterns = ["\\.setup$", "\\._"]
        if class_skip_patterns:
            self.class_skip_patterns = class_skip_patterns
        else:
            self.class_skip_patterns = []
        self.package_name = package_name
        self.package_skip_patterns = package_skip_patterns
        self.module_skip_patterns = module_skip_patterns

    def get_package_name(self):
        return self._package_name

    def set_package_name(self, package_name):
        """Set package_name"""
        # It's also possible to imagine caching the module parsing here
        self._package_name = package_name
        self.root_module = __import__(package_name)
        self.root_path = self.root_module.__path__[0]

    package_name = property(
        get_package_name, set_package_name, None, "get/set package_name"
    )

    def _get_object_name(self, line):
        name = line.split()[1].split("(")[0].strip()
        # in case we have classes which are not derived from object
        # ie. old style classes
        return name.rstrip(":")

    def _uri2path(self, uri):
        """Convert uri to absolute filepath

        Parameters
        ----------
        uri : string
            URI of python module to return path for

        Returns
        -------
        path : None or string
            Returns None if there is no valid path for this URI
            Otherwise returns absolute file system path for URI

        """
        if uri == self.package_name:
            return os.path.join(self.root_path, "__init__.py")
        path = uri.replace(".", os.path.sep)
        path = path.replace(self.package_name + os.path.sep, "")
        path = os.path.join(self.root_path, path)
        # XXX maybe check for extensions as well?
        if os.path.exists(path + ".py"):  # file
            path += ".py"
        elif os.path.exists(os.path.join(path, "__init__.py")):
            path = os.path.join(path, "__init__.py")
        else:
            return None
        return path

    def _path2uri(self, dirpath):
        """ Convert directory path to uri """
        relpath = dirpath.replace(self.root_path, self.package_name)
        if relpath.startswith(os.path.sep):
            relpath = relpath[1:]
        return relpath.replace(os.path.sep, ".")

    def _parse_module(self, uri):
        """ Parse module defined in *uri* """
        filename = self._uri2path(uri)
        if filename is None:
            # nothing that we could handle here.
            return ([], [])
        f = open(filename, "rt")
        functions, classes = self._parse_lines(f, uri)
        f.close()
        return functions, classes

    def _parse_lines(self, linesource, module):
        """ Parse lines of text for functions and classes """
        functions = []
        classes = []
        for line in linesource:
            if line.startswith("def ") and line.count("("):
                # exclude private stuff
                name = self._get_object_name(line)
                if not name.startswith("_"):
                    functions.append(name)
            elif line.startswith("class "):
                # exclude private stuff
                name = self._get_object_name(line)
                if not name.startswith("_") and self._survives_exclude(
                    ".".join((module, name)), "class"
                ):
                    classes.append(name)
            else:
                pass
        functions.sort()
        classes.sort()
        return functions, classes

    @classmethod
    def _normalize_repr(cls, value):
        if isinstance(value, list):
            return "[{}]".format(", ".join(map(cls._normalize_repr, value)))
        if isinstance(value, tuple):
            if len(value) == 1:
                return "({},)".format(cls._normalize_repr(value[0]))
            return "({})".format(", ".join(map(cls._normalize_repr, value)))
        if isinstance(value, (str, bytes)):
            value = repr(value)
            if value[0] not in ('"', "'"):
                value = value[1:]
        else:
            value = repr(value)
        return value

    def test_specs(self, uri):
        """Check input and output specs in an uri

        Parameters
        ----------
        uri : string
            python location of module - e.g 'sphinx.builder'

        Returns
        -------
        """
        from nipype.interfaces.base import BaseInterface

        # get the names of all classes and functions
        _, classes = self._parse_module(uri)
        if not classes:
            # print 'WARNING: Empty -',uri  # dbg
            return None

        # Make a shorter version of the uri that omits the package name for
        # titles
        allowed_keys = [
            "desc",
            "genfile",
            "xor",
            "requires",
            "desc",
            "nohash",
            "argstr",
            "position",
            "mandatory",
            "copyfile",
            "usedefault",
            "sep",
            "hash_files",
            "deprecated",
            "new_name",
            "min_ver",
            "max_ver",
            "name_source",
            "name_template",
            "keep_extension",
            "units",
            "output_name",
            "extensions",
        ]
        in_built = [
            "type",
            "copy",
            "parent",
            "instance_handler",
            "comparison_mode",
            "array",
            "default",
            "editor",
        ]
        bad_specs = []
        for c in classes:
            __import__(uri)
            try:
                with warnings.catch_warnings():
                    warnings.simplefilter("ignore")
                    classinst = sys.modules[uri].__dict__[c]
            except Exception:
                continue

            if not issubclass(classinst, BaseInterface):
                continue
            testdir = os.path.join(*(uri.split(".")[:-1] + ["tests"]))
            if not os.path.exists(testdir):
                os.makedirs(testdir)
            nonautotest = os.path.join(testdir, "test_%s.py" % c)
            testfile = os.path.join(testdir, "test_auto_%s.py" % c)
            if os.path.exists(testfile):
                os.unlink(testfile)

            if not os.path.exists(nonautotest):
                cmd = [
                    "# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT",
                    "from ..%s import %s" % (uri.split(".")[-1], c),
                    "",
                ]
                cmd.append("\ndef test_%s_inputs():" % c)
                input_fields = ""
                for traitname, trait in sorted(
                    classinst.input_spec().traits(transient=None).items()
                ):
                    input_fields += "%s=dict(" % traitname
                    for key, value in sorted(trait.__dict__.items()):
                        if key in in_built or key == "desc":
                            continue
                        input_fields += "%s=%s,\n    " % (
                            key,
                            self._normalize_repr(value),
                        )
                    input_fields += "),\n    "
                cmd += ["    input_map = dict(%s)" % input_fields]
                cmd += ["    inputs = %s.input_spec()" % c]
                cmd += [
                    """
    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value"""
                ]

                fmt_cmd = black.format_str("\n".join(cmd), mode=black.FileMode())
                with open(testfile, "wt") as fp:
                    fp.writelines(fmt_cmd)
            else:
                print("%s has nonautotest" % c)

            for traitname, trait in sorted(
                classinst.input_spec().traits(transient=None).items()
            ):
                for key in sorted(trait.__dict__):
                    if key in in_built:
                        continue
                    parent_metadata = []
                    if "parent" in trait.__dict__:
                        parent_metadata = list(getattr(trait, "parent").__dict__.keys())
                    if (
                        key
                        not in allowed_keys
                        + classinst._additional_metadata
                        + parent_metadata
                    ):
                        bad_specs.append([uri, c, "Inputs", traitname, key])
                    if (
                        key == "mandatory"
                        and trait.mandatory is not None
                        and not trait.mandatory
                    ):
                        bad_specs.append(
                            [uri, c, "Inputs", traitname, "mandatory=False"]
                        )
                    if key == "usedefault" and trait.__dict__[key] == False:
                        bad_specs.append(
                            [uri, c, "Inputs", traitname, "usedefault=False"]
                        )
                # checking if traits that have default_value different that the trits default one
                # also have `usedefault` specified;
                # excluding TraitCompound
                # excluding Enum: always has default value (the first value)
                # excluding Tuple: takes tuple of inner traits default values as default, but doesn't use it
                # for Range assuming that if default == low, it's likely that usedefault should be False
                # (for Range traits takes low as a default default
                if (
                    trait.trait_type.__class__.__name__
                    not in ["TraitCompound", "Tuple", "Enum"]
                    and trait.default
                    and "usedefault" not in trait.__dict__
                    and "requires" not in trait.__dict__
                    and "xor" not in trait.__dict__
                ):
                    if (
                        trait.trait_type.__class__.__name__ == "Range"
                        and trait.default == trait.trait_type._low
                    ):
                        continue
                    bad_specs.append(
                        [
                            uri,
                            c,
                            "Inputs",
                            traitname,
                            "default value is set, no value for usedefault",
                        ]
                    )

            if not classinst.output_spec:
                continue

            if not os.path.exists(nonautotest):
                cmd = ["\ndef test_%s_outputs():" % c]
                input_fields = ""
                for traitname, trait in sorted(
                    classinst.output_spec().traits(transient=None).items()
                ):
                    input_fields += "%s=dict(" % traitname
                    for key, value in sorted(trait.__dict__.items()):
                        if key in in_built or key == "desc":
                            continue
                        input_fields += "%s=%s,\n    " % (
                            key,
                            self._normalize_repr(value),
                        )
                    input_fields += "),\n    "
                cmd += ["    output_map = dict(%s)" % input_fields]
                cmd += ["    outputs = %s.output_spec()" % c]
                cmd += [
                    """
    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value"""
                ]
                fmt_cmd = black.format_str("\n".join(cmd), mode=black.FileMode())
                with open(testfile, "at") as fp:
                    fp.writelines("\n\n" + fmt_cmd)

            for traitname, trait in sorted(
                classinst.output_spec().traits(transient=None).items()
            ):
                for key in sorted(trait.__dict__):
                    if key in in_built:
                        continue
                    parent_metadata = []
                    if "parent" in trait.__dict__:
                        parent_metadata = list(getattr(trait, "parent").__dict__.keys())
                    if (
                        key
                        not in allowed_keys
                        + classinst._additional_metadata
                        + parent_metadata
                    ):
                        bad_specs.append([uri, c, "Outputs", traitname, key])
        return bad_specs

    def _survives_exclude(self, matchstr, match_type):
        """ Returns True if *matchstr* does not match patterns

        ``self.package_name`` removed from front of string if present

        Examples
        --------
        >>> dw = ApiDocWriter('sphinx')
        >>> dw._survives_exclude('sphinx.okpkg', 'package')
        True
        >>> dw.package_skip_patterns.append('^\\.badpkg$')
        >>> dw._survives_exclude('sphinx.badpkg', 'package')
        False
        >>> dw._survives_exclude('sphinx.badpkg', 'module')
        True
        >>> dw._survives_exclude('sphinx.badmod', 'module')
        True
        >>> dw.module_skip_patterns.append('^\\.badmod$')
        >>> dw._survives_exclude('sphinx.badmod', 'module')
        False
        """
        if match_type == "module":
            patterns = self.module_skip_patterns
        elif match_type == "package":
            patterns = self.package_skip_patterns
        elif match_type == "class":
            patterns = self.class_skip_patterns
        else:
            raise ValueError('Cannot interpret match type "%s"' % match_type)
        # Match to URI without package name
        L = len(self.package_name)
        if matchstr[:L] == self.package_name:
            matchstr = matchstr[L:]
        for pat in patterns:
            try:
                pat.search
            except AttributeError:
                pat = re.compile(pat)
            if pat.search(matchstr):
                return False
        return True

    def discover_modules(self):
        """ Return module sequence discovered from ``self.package_name``


        Parameters
        ----------
        None

        Returns
        -------
        mods : sequence
            Sequence of module names within ``self.package_name``

        Examples
        --------
        """
        modules = [self.package_name]
        # raw directory parsing
        for dirpath, dirnames, filenames in os.walk(self.root_path):
            # Check directory names for packages
            root_uri = self._path2uri(os.path.join(self.root_path, dirpath))
            for dirname in dirnames[:]:  # copy list - we modify inplace
                package_uri = ".".join((root_uri, dirname))
                if self._uri2path(package_uri) and self._survives_exclude(
                    package_uri, "package"
                ):
                    modules.append(package_uri)
                else:
                    dirnames.remove(dirname)
            # Check filenames for modules
            for filename in filenames:
                module_name = filename[:-3]
                module_uri = ".".join((root_uri, module_name))
                if self._uri2path(module_uri) and self._survives_exclude(
                    module_uri, "module"
                ):
                    modules.append(module_uri)
        return sorted(modules)

    def check_modules(self):
        # write the list
        modules = self.discover_modules()
        checked_modules = []
        for m in modules:
            bad_specs = self.test_specs(m)
            if bad_specs:
                checked_modules.extend(bad_specs)
        for bad_spec in checked_modules:
            print(":".join(bad_spec))


if __name__ == "__main__":
    os.environ["NIPYPE_NO_ET"] = "1"
    package = "nipype"
    ic = InterfaceChecker(package)
    # Packages that should not be included in generated API docs.
    ic.package_skip_patterns += [
        "\.external$",
        "\.fixes$",
        "\.utils$",
        "\.pipeline",
        "\.testing",
        "\.caching",
        "\.workflows",
    ]
    """
    # Modules that should not be included in generated API docs.
    ic.module_skip_patterns += ['\.version$',
                                '\.interfaces\.base$',
                                '\.interfaces\.matlab$',
                                '\.interfaces\.rest$',
                                '\.interfaces\.pymvpa$',
                                '\.interfaces\.slicer\.generate_classes$',
                                '\.interfaces\.spm\.base$',
                                '\.interfaces\.traits',
                                '\.pipeline\.alloy$',
                                '\.pipeline\.s3_node_wrapper$',
                                '.\testing',
                                       ]
    ic.class_skip_patterns += ['AFNI',
                               'ANTS',
                               'FSL',
                               'FS',
                               'Info',
                               '^SPM',
                               'Tester',
                               'Spec$',
                               'Numpy',
                               'NipypeTester',
                                      ]
    """
    ic.check_modules()



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..convert import Trackvis2Camino


def test_Trackvis2Camino_inputs():
    input_map = dict(
        append_file=dict(
            argstr="-a %s",
            extensions=None,
            position=2,
        ),
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        out_file=dict(
            argstr="-o %s",
            extensions=None,
            genfile=True,
            position=2,
        ),
    )
    inputs = Trackvis2Camino.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Trackvis2Camino_outputs():
    output_map = dict(
        camino=dict(
            extensions=None,
        ),
    )
    outputs = Trackvis2Camino.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Test testing utilities
"""

import os
import warnings
import subprocess
from unittest.mock import patch, MagicMock
from unittest import SkipTest
from nipype.testing.utils import TempFATFS


def test_tempfatfs():
    try:
        fatfs = TempFATFS()
    except (IOError, OSError):
        raise SkipTest("Cannot mount FAT filesystems with FUSE")
    with fatfs as tmp_dir:
        assert os.path.exists(tmp_dir)


@patch(
    "subprocess.check_call",
    MagicMock(side_effect=subprocess.CalledProcessError("", "")),
)
def test_tempfatfs_calledprocesserror():
    try:
        TempFATFS()
    except IOError as e:
        assert isinstance(e, IOError)
        assert isinstance(e.__cause__, subprocess.CalledProcessError)
    else:
        assert False


@patch("subprocess.check_call", MagicMock())
@patch("subprocess.Popen", MagicMock(side_effect=OSError()))
def test_tempfatfs_oserror():
    try:
        TempFATFS()
    except IOError as e:
        assert isinstance(e, IOError)
        assert isinstance(e.__cause__, OSError)
    else:
        assert False



#!/usr/bin/env python3
from runpy import run_path
from pathlib import Path

repo_root = Path(__file__).parent.parent
info_file = repo_root / "nipype" / "info.py"
reqs = repo_root / "requirements.txt"

info = run_path(info_file)
requirements = info["REQUIRES"]

script_name = Path(__file__).relative_to(repo_root)

lines = [f"# Auto-generated by {script_name}", ""]

# Write requirements
lines[1:-1] = requirements
reqs.write_text("\n".join(lines))



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import SphericalAverage


def test_SphericalAverage_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        erode=dict(
            argstr="-erode %d",
        ),
        fname=dict(
            argstr="%s",
            mandatory=True,
            position=-5,
        ),
        hemisphere=dict(
            argstr="%s",
            mandatory=True,
            position=-4,
        ),
        in_average=dict(
            argstr="%s",
            genfile=True,
            position=-2,
        ),
        in_orig=dict(
            argstr="-orig %s",
            extensions=None,
        ),
        in_surf=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        subject_id=dict(
            argstr="-o %s",
            mandatory=True,
        ),
        subjects_dir=dict(),
        threshold=dict(
            argstr="-t %.1f",
        ),
        which=dict(
            argstr="%s",
            mandatory=True,
            position=-6,
        ),
    )
    inputs = SphericalAverage.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SphericalAverage_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = SphericalAverage.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..base import DipyDiffusionInterface


def test_DipyDiffusionInterface_inputs():
    input_map = dict(
        b0_thres=dict(
            usedefault=True,
        ),
        in_bval=dict(
            extensions=None,
            mandatory=True,
        ),
        in_bvec=dict(
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        out_prefix=dict(),
    )
    inputs = DipyDiffusionInterface.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
import sys
from textwrap import dedent


if __name__ == "__main__":
    print(dedent("""Nipype examples have been moved to niflow-nipype1-examples.

Install with: pip install niflow-nipype1-examples"""))
    if sys.argv[1:]:
        print("Run this command with: niflow-nipype1-examples " + " ".join(sys.argv[1:]))
    sys.exit(1)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Notes


def test_Notes_inputs():
    input_map = dict(
        add=dict(
            argstr='-a "%s"',
        ),
        add_history=dict(
            argstr='-h "%s"',
            xor=["rep_history"],
        ),
        args=dict(
            argstr="%s",
        ),
        delete=dict(
            argstr="-d %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
        ),
        outputtype=dict(),
        rep_history=dict(
            argstr='-HH "%s"',
            xor=["add_history"],
        ),
        ses=dict(
            argstr="-ses",
        ),
    )
    inputs = Notes.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Notes_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Notes.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- coding: utf-8; mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set fileencoding=utf-8 ft=python sts=4 ts=4 sw=4 et:
#
# nipype documentation build configuration file, created by
# sphinx-quickstart on Mon Jul 20 12:30:18 2009.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import os
from pathlib import Path
from tempfile import TemporaryDirectory
import shutil
import sys
from packaging.version import Version
import nipype
import subprocess as sp

# Disable etelemetry during doc builds
os.environ["NIPYPE_NO_ET"] = "1"

conf_py = Path(__file__)

example_dir = conf_py.parent / "users" / "examples"
shutil.rmtree(example_dir, ignore_errors=True)
example_dir.mkdir(parents=True)
python_dir = conf_py.parent / "_static" / "python"
shutil.rmtree(python_dir, ignore_errors=True)

ex2rst = str(conf_py.parent.parent / "tools" / "ex2rst")

with TemporaryDirectory() as tmpdir:
    sp.run(
        [
            "git",
            "clone",
            "--depth",
            "1",
            "https://github.com/niflows/nipype1-examples.git",
            tmpdir,
        ],
        check=True,
    )
    source_dir = Path(tmpdir) / "package" / "niflow" / "nipype1" / "examples"
    shutil.copytree(
        source_dir,
        python_dir,
        ignore=lambda src, names: [n for n in names if n.endswith(".ipynb")],
    )

sp.run(
    [
        sys.executable,
        ex2rst,
        "--outdir",
        str(example_dir),
        str(python_dir),
        "-x",
        str(python_dir / "test_spm.py"),
        "-x",
        str(python_dir / "__init__.py"),
        "-x",
        str(python_dir / "cli.py"),
    ],
    check=True,
)
sp.run(
    [
        sys.executable,
        ex2rst,
        "--outdir",
        str(example_dir),
        str(python_dir / "frontiers_paper"),
    ],
    check=True,
)


# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
# sys.path.append(os.path.abspath('sphinxext'))

# -- General configuration -----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.graphviz",
    "sphinx.ext.mathjax",
    "sphinx.ext.inheritance_diagram",
    "sphinx.ext.todo",
    "sphinxcontrib.apidoc",
    "matplotlib.sphinxext.plot_directive",
    "nbsphinx",
    "nipype.sphinxext.plot_workflow",
    "nipype.sphinxext.apidoc",
    "nipype.sphinxext.documenter",
]

autodoc_mock_imports = [
    "matplotlib",
    "nilearn",
    "nipy",
    "nitime",
    "numpy",
    "pandas",
    "seaborn",
    "skimage",
    "svgutils",
    "transforms3d",
    "tvtk",
    "vtk",
]

# Accept custom section names to be parsed for numpy-style docstrings
# of parameters.
# Requires pinning sphinxcontrib-napoleon to a specific commit while
# https://github.com/sphinx-contrib/napoleon/pull/10 is merged.
napoleon_use_param = False
napoleon_custom_sections = [
    ("Inputs", "Parameters"),
    ("Outputs", "Parameters"),
    ("Attributes", "Parameters"),
    ("Mandatory Inputs", "Parameters"),
    ("Optional Inputs", "Parameters"),
]


on_rtd = os.environ.get("READTHEDOCS") == "True"
if on_rtd:
    extensions.append("readthedocs_ext.readthedocs")

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix of source filenames.
source_suffix = ".rst"

# The encoding of source files.
# source_encoding = 'utf-8'

# The master toctree document.
master_doc = "index"

# General information about the project.
project = u"nipype"
copyright = u"2009-20, Neuroimaging in Python team"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = Version(nipype.__version__).public
# The full version, including alpha/beta/rc tags.
release = nipype.__version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
today_fmt = "%B %d, %Y, %H:%M PDT"

# List of documents that shouldn't be included in the build.
unused_docs = ["api/generated/gen"]

# List of directories, relative to source directory, that shouldn't be searched
# for source files.
exclude_trees = ["_build"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# The reST default role (used for this markup: `text`) to use for all documents.
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []

# -- Sphinxext configuration ---------------------------------------------------

# Set attributes for layout of inheritance diagrams
inheritance_graph_attrs = dict(
    rankdir="LR", size='"6.0, 8.0"', fontsize=14, ratio="compress"
)
inheritance_node_attrs = dict(
    shape="ellipse", fontsize=14, height=0.75, color="dodgerblue1", style="filled"
)

# Flag to show todo items in rendered output
todo_include_todos = True

# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme = "sphinxdoc"

# The style sheet to use for HTML and HTML Help pages. A file of that name
# must exist either in Sphinx' static/ path, or in one of the custom paths
# given in html_static_path.
html_style = "nipype.css"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = "nipy pipeline and interfaces package"

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
# html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# Content template for the index page.
html_index = "index.html"

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
html_sidebars = {
    "**": ["gse.html", "localtoc.html", "sidebar_versions.html", "indexsidebar.html"],
    "searchresults": ["sidebar_versions.html", "indexsidebar.html"],
    "version": [],
}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {'index': 'index.html'}

# If false, no module index is generated.
# html_use_modindex = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = "nipypedoc"


# -- Options for LaTeX output --------------------------------------------------

# The paper size ('letter' or 'a4').
# latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
# latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
    (
        "interfaces",
        "interfaces.tex",
        "Nipype Interfaces Documentation",
        "Neuroimaging in Python team",
        "manual",
    ),
    # ('developers', 'developers.tex', 'Nipype API',
    #  'Neuroimaging in Python team', 'manual'),
    (
        "examples",
        "examples.tex",
        "Nipype Examples",
        "Neuroimaging in Python team",
        "manual",
    ),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# Additional stuff for the LaTeX preamble.
# latex_preamble = ''

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_use_modindex = True

# -- apidoc extension configuration ------------------------------------------
apidoc_module_dir = "../nipype"
apidoc_output_dir = "api/generated"
apidoc_excluded_paths = [
    "*/tests/*",
    "tests/*",
    "external/*",
    "fixes/*",
    "scripts/*",
    "testing/*",
    "workflows/*",
    "conftest.py",
    "info.py",
    "pkg_info.py",
    "refs.py",
]
apidoc_separate_modules = True
apidoc_extra_args = ["--module-first", "-d 1", "-T"]

# -- Options for intersphinx extension ---------------------------------------

# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {"http://docs.python.org/": None}



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import Automask


def test_Automask_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        brain_file=dict(
            argstr="-apply_prefix %s",
            extensions=None,
            name_source="in_file",
            name_template="%s_masked",
        ),
        clfrac=dict(
            argstr="-clfrac %s",
        ),
        dilate=dict(
            argstr="-dilate %s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        erode=dict(
            argstr="-erode %s",
        ),
        in_file=dict(
            argstr="%s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file",
            name_template="%s_mask",
        ),
        outputtype=dict(),
    )
    inputs = Automask.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Automask_outputs():
    output_map = dict(
        brain_file=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Automask.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



#!/usr/bin/env python3
"""Update and sort the creators list of the zenodo record."""
import git
import json
from subprocess import run, PIPE
from pathlib import Path
from fuzzywuzzy import fuzz, process


def decommify(name):
    return " ".join(name.split(", ")[::-1])


# These names should go last
CREATORS_LAST = ["Krzysztof J. Gorgolewski", "Satrajit Ghosh"]

# Contributors that have requested not to be cited (or bothered)
BLACKLIST = {"Jonathan R. Williford"}

if __name__ == "__main__":
    git_root = Path(git.Repo(".", search_parent_directories=True).working_dir)
    zenodo_file = git_root / ".zenodo.json"

    zenodo = json.loads(zenodo_file.read_text()) if zenodo_file.exists() else {}

    creator_map = {
        decommify(creator["name"]): creator for creator in zenodo.get("creators", [])
    }

    shortlog = run(["git", "shortlog", "-ns"], stdout=PIPE)
    commit_counts = dict(
        line.split("\t", 1)[::-1]
        for line in shortlog.stdout.decode().split("\n")
        if line
    )

    existing_creators = set(creator_map.keys())

    committers = []

    # Stable sort:
    # Number of commits in descending order
    # Ties broken by alphabetical order of first name
    for committer, _ in sorted(commit_counts.items(), key=lambda x: (-int(x[1]), x[0])):
        matches = process.extract(
            committer, creator_map.keys(), scorer=fuzz.token_sort_ratio, limit=2
        )
        match, score = matches[0]
        if score <= 80:
            if committer not in BLACKLIST:
                print("No entry to sort:", committer)
            continue
        existing_creators.discard(match)
        committers.append(match)

    for unmatched in sorted(existing_creators):
        print("No matching commits:", unmatched)
        # Keep the entries to avoid removing people for bad matching
        committers.append(unmatched)

    for last_author in CREATORS_LAST:
        if committers[-1] != last_author:
            committers.remove(last_author)
            committers.append(last_author)

    creators = [
        creator_map.get(committer, {"name": committer}) for committer in committers
    ]

    zenodo["creators"] = creators

    zenodo_file.write_text("%s\n" % json.dumps(zenodo, indent=2))
